@@ -1,7 +1,7 @@
 // SPDX-License-Identifier: (Apache-2.0 OR MIT)
 
-use crate::exc::*;
 use crate::opt::*;
+use crate::serialize::error::*;
 use crate::serialize::serializer::*;
 
 use serde::ser::{Serialize, Serializer};
@@ -34,8 +34,9 @@ impl DefaultSerializer {
     }
 }
 
-impl<'p> Serialize for DefaultSerializer {
+impl Serialize for DefaultSerializer {
     #[inline(never)]
+    #[cfg_attr(feature = "optimize", optimize(size))]
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
